
# q-expansion of newform 836.2.i.c, downloaded from the LMFDB on 11 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 836
weight = 2
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[419, [1, 0]], [761, [1, 0]], [705, [0, -1]]]
aps_data = [[0, 0], [0, 0], [-3, 3], [-3, 0], [-1, 0], [0, -2], [2, -2], [3, -5], [0, -8], [0, 0], [-10, 0], [-3, 0], [4, -4], [-4, 4], [0, 6], [0, -5], [-4, 4], [0, 12], [0, -12], [-2, 2], [-6, 6], [-5, 5], [9, 0], [0, -6], [-5, 5], [0, -8], [-4, 0], [3, 0], [4, -4], [-14, 0], [0, -16], [0, 0], [0, 3], [0, 7], [-16, 16], [-9, 0], [-3, 3], [4, 0], [0, -25], [12, -12], [18, 0], [0, 23], [12, 0], [-26, 26], [2, 0], [0, 20], [-23, 23], [2, -2], [11, 0], [-23, 0], [-10, 10], [-21, 0], [0, 8], [0, -12], [0, 13], [-1, 1], [-21, 21], [-7, 7], [2, 0], [0, -18], [-11, 11], [24, 0], [-11, 11], [-8, 0], [0, -19], [0, -10], [-14, 0], [26, -26], [29, -29], [10, 0], [-9, 0], [5, -5], [0, -16], [34, 0], [16, 0], [-36, 36], [0, -31], [-15, 15], [-18, 18], [0, -22], [14, 0], [37, -37], [0, -33], [0, -2], [-37, 37], [0, -16], [-13, 0], [-32, 0], [36, -36], [-40, 0], [4, 0], [0, 32], [0, 0], [1, -1], [-2, 2], [0, 12], [0, -1], [-15, 0], [0, -3], [0, 2], [0, -17], [0, -30], [29, 0], [34, 0], [-11, 0], [17, 0], [2, -2], [0, 34], [0, 30], [-24, 0], [40, 0], [8, -8], [0, -33], [32, 0], [0, 34], [49, -49], [-34, 34], [-6, 0], [-10, 0], [0, -35], [0, -43], [-24, 0], [50, 0], [6, 0], [-28, 0], [6, -6], [0, 35], [-34, 34], [14, -14], [30, 0], [8, -8], [11, -11], [0, 22], [-26, 26], [28, 0], [0, -26], [0, 6], [-11, 0], [43, 0], [24, 0], [0, 35], [0, 54], [0, -20], [0, -36], [-14, 0], [-54, 54], [-42, 42], [0, 0], [0, 12], [56, 0], [54, -54], [26, 0], [0, -2], [0, -21], [-4, 4], [-12, 0], [-3, 0], [46, -46], [0, -8], [0, -24], [32, -32], [8, -8], [-35, 35], [-48, 48], [9, 0], [-12, 12], [58, -58], [0, 20]]
